@@ -74,7 +74,6 @@
 #include "lsquic_min_heap.h"
 #include "lsquic_http1x_if.h"
 #include "lsquic_parse_common.h"
-#include "lsquic_h3_prio.h"
 #include "lsquic_handshake.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_ENGINE
@@ -165,12 +164,6 @@ force_close_conn (lsquic_engine_t *engine, lsquic_conn_t *conn);
                         |LSCONN_ATTQ)
 
 
-struct cert_susp_head
-{
-    TAILQ_HEAD(, lsquic_conn)   csh_conns;
-    struct lsquic_hash_elem     csh_hash_el;
-    char                        csh_sni[0];
-};
 
 
 struct cid_update_batch
@@ -237,7 +230,6 @@ struct lsquic_engine
     lsquic_time_t                      deadline;
     lsquic_time_t                      resume_sending_at;
     unsigned                           mini_conns_count;
-    struct lsquic_hash                *suspended_sni_heads;
     struct lsquic_purga               *purga;
 #if LSQUIC_CONN_STATS
     struct {
@@ -327,7 +319,6 @@ lsquic_engine_init_settings (struct lsquic_engine_settings *settings,
     settings->es_qpack_dec_max_blocked = LSQUIC_DF_QPACK_DEC_MAX_BLOCKED;
     settings->es_qpack_enc_max_size = LSQUIC_DF_QPACK_ENC_MAX_SIZE;
     settings->es_qpack_enc_max_blocked = LSQUIC_DF_QPACK_ENC_MAX_BLOCKED;
-    settings->es_h3_placeholders = LSQUIC_DF_H3_PLACEHOLDERS;
     settings->es_allow_migration = LSQUIC_DF_ALLOW_MIGRATION;
 }
 
@@ -338,8 +329,6 @@ lsquic_engine_check_settings (const struct lsquic_engine_settings *settings,
                               unsigned flags,
                               char *err_buf, size_t err_buf_sz)
 {
-    unsigned sum;
-
     if (settings->es_cfcw < LSQUIC_MIN_FCW ||
         settings->es_sfcw < LSQUIC_MIN_FCW)
     {
@@ -391,19 +380,6 @@ lsquic_engine_check_settings (const struct lsquic_engine_settings *settings,
         return -1;
     }
 
-    sum = settings->es_init_max_streams_bidi
-        + settings->es_init_max_streams_uni
-        + settings->es_h3_placeholders;
-    if (sum > H3_PRIO_MAX_ELEMS)
-    {
-        if (err_buf)
-            snprintf(err_buf, err_buf_sz, "Combined number of streams and "
-                "placeholders (%u) is greater than the maximum supported "
-                "number of elements in the HTTP/3 priority tree (%u)",
-                sum, H3_PRIO_MAX_ELEMS);
-        return -1;
-    }
-
     if (settings->es_cc_algo > 2)
     {
         if (err_buf)
@@ -581,7 +557,6 @@ lsquic_engine_new (unsigned flags,
     engine->attq = attq_create();
     eng_hist_init(&engine->history);
     engine->batch_size = INITIAL_OUT_BATCH_SIZE;
-    engine->suspended_sni_heads = lsquic_hash_create();
     if (engine->pub.enp_settings.es_honor_prst)
     {
         engine->pub.enp_srst_hash = lsquic_hash_create();
@@ -659,20 +634,6 @@ shrink_batch_size (struct lsquic_engine *engine)
 }
 
 
-static void
-delete_susp_head (struct lsquic_engine *engine, struct cert_susp_head *head)
-{
-    struct lsquic_hash_elem *el;
-
-    el = lsquic_hash_find(engine->suspended_sni_heads, head->csh_sni,
-                          strlen(head->csh_sni));
-    assert(el);
-    assert(head == lsquic_hashelem_getdata(el));
-    lsquic_hash_erase(engine->suspended_sni_heads, el);
-    free(head);
-}
-
-
 struct cce_cid_iter
 {
     const struct lsquic_conn   *conn;
@@ -744,14 +705,6 @@ destroy_conn (struct lsquic_engine *engine, struct lsquic_conn *conn,
     lsquic_time_t drain_time;
     struct purga_el *puel;
 
-    if (conn->cn_cert_susp_head)
-    {
-        TAILQ_REMOVE(&conn->cn_cert_susp_head->csh_conns, conn,
-                     cn_next_susp_cert);
-        if (TAILQ_EMPTY(&conn->cn_cert_susp_head->csh_conns))
-            delete_susp_head(engine, conn->cn_cert_susp_head);
-        conn->cn_cert_susp_head = NULL;
-    }
     engine->mini_conns_count -= !!(conn->cn_flags & LSCONN_MINI);
     if (engine->purga
         /* Blacklist all CIDs except for promoted mini connections */
@@ -985,8 +938,7 @@ schedule_req_packet (struct lsquic_engine *engine, enum packet_req_type type,
         LSQ_DEBUGC("scheduled %s packet for cid %"CID_FMT,
                     lsquic_preqt2str[type], CID_BITS(&packet_in->pi_conn_id));
     else
-        LSQ_DEBUG("cannot schedule %s packet: out of packet request objects",
-                    lsquic_preqt2str[type]);
+        LSQ_DEBUG("cannot schedule %s packet", lsquic_preqt2str[type]);
 }
 
 
@@ -1411,7 +1363,6 @@ lsquic_engine_destroy (lsquic_engine_t *engine)
     if (engine->flags & ENG_LOSE_PACKETS)
         regfree(&engine->lose_packets_re);
 #endif
-    lsquic_hash_destroy(engine->suspended_sni_heads);
     if (engine->pub.enp_tokgen)
         lsquic_tg_destroy(engine->pub.enp_tokgen);
 #if LSQUIC_CONN_STATS
