@@ -1940,14 +1940,19 @@ mini_conn_ci_record_addrs (struct lsquic_conn *lconn, void *peer_ctx,
 {
     struct mini_conn *mc = (struct mini_conn *) lconn;
     struct lsquic_packet_out *packet_out;
+    size_t len;
+
 
     if (NP_IS_IPv6(&mc->mc_path) != (AF_INET6 == peer_sa->sa_family))
         TAILQ_FOREACH(packet_out, &mc->mc_packets_out, po_next)
             if ((packet_out->po_flags & (PO_SENT|PO_ENCRYPTED)) == PO_ENCRYPTED)
                 return_enc_data(mc, packet_out);
 
-    memcpy(mc->mc_path.np_peer_addr, peer_sa, sizeof(mc->mc_path.np_peer_addr));
-    memcpy(mc->mc_path.np_local_addr, local_sa, sizeof(mc->mc_path.np_local_addr));
+    len = local_sa->sa_family == AF_INET ? sizeof(struct sockaddr_in)
+                                                : sizeof(struct sockaddr_in6);
+
+    memcpy(mc->mc_path.np_peer_addr, peer_sa, len);
+    memcpy(mc->mc_path.np_local_addr, local_sa, len);
     mc->mc_path.np_peer_ctx = peer_ctx;
     return 0;
 }
