@@ -40,7 +40,6 @@
 #include "lsquic_conn_public.h"
 #include "lsquic_cong_ctl.h"
 #include "lsquic_enc_sess.h"
-#include "lsquic_h3_prio.h"
 #include "lsquic_hash.h"
 #include "lsquic_malo.h"
 
@@ -312,12 +311,12 @@ lsquic_send_ctl_init (lsquic_send_ctl_t *ctl, struct lsquic_alarmset *alset,
     ctl->sc_alset = alset;
     ctl->sc_ver_neg = ver_neg;
     ctl->sc_conn_pub = conn_pub;
-    assert(!(flags & ~(SC_IETF|SC_NSTP)));
+    assert(!(flags & ~(SC_IETF|SC_NSTP|SC_ECN)));
     ctl->sc_flags = flags;
     send_ctl_pick_initial_packno(ctl);
     if (enpub->enp_settings.es_pace_packets)
         ctl->sc_flags |= SC_PACE;
-    if ((flags & SC_IETF) && enpub->enp_settings.es_ecn)
+    if (flags & SC_ECN)
         ctl->sc_ecn = ECN_ECT0;
     else
         ctl->sc_ecn = ECN_NOT_ECT;
@@ -591,7 +590,8 @@ lsquic_send_ctl_sent_packet (lsquic_send_ctl_t *ctl,
     ++ctl->sc_stats.n_total_sent;
 #endif
     if (ctl->sc_ci->cci_sent)
-        ctl->sc_ci->cci_sent(CGP(ctl), packet_out, ctl->sc_n_in_flight_all);
+        ctl->sc_ci->cci_sent(CGP(ctl), packet_out, ctl->sc_n_in_flight_all,
+                                            ctl->sc_flags & SC_APP_LIMITED);
     lsquic_send_ctl_sanity_check(ctl);
     return 0;
 }
@@ -772,6 +772,16 @@ send_ctl_handle_lost_packet (lsquic_send_ctl_t *ctl,
     if (ctl->sc_ci->cci_lost)
         ctl->sc_ci->cci_lost(CGP(ctl), packet_out, packet_sz);
 
+    /* This is a client-only check, server check happens in mini conn */
+    if (send_ctl_ecn_on(ctl)
+            && 0 == ctl->sc_ecn_total_acked[PNS_INIT]
+                && HETY_INITIAL == packet_out->po_header_type
+                    && 3 == packet_out->po_packno)
+    {
+        LSQ_DEBUG("possible ECN black hole during handshake, disable ECN");
+        ctl->sc_ecn = ECN_NOT_ECT;
+    }
+
     if (packet_out->po_frame_types & ctl->sc_retx_frames)
     {
         LSQ_DEBUG("lost retransmittable packet %"PRIu64,
@@ -1223,6 +1233,7 @@ lsquic_send_ctl_cleanup (lsquic_send_ctl_t *ctl)
     }
     if (ctl->sc_flags & SC_PACE)
         pacer_cleanup(&ctl->sc_pacer);
+    ctl->sc_ci->cci_cleanup(CGP(ctl));
 #if LSQUIC_SEND_STATS
     LSQ_NOTICE("stats: n_total_sent: %u; n_resent: %u; n_delayed: %u",
         ctl->sc_stats.n_total_sent, ctl->sc_stats.n_resent,
@@ -1291,6 +1302,38 @@ lsquic_send_ctl_can_send (lsquic_send_ctl_t *ctl)
 }
 
 
+/* Like lsquic_send_ctl_can_send(), but no mods */
+static int
+send_ctl_could_send (const struct lsquic_send_ctl *ctl)
+{
+    uint64_t cwnd;
+    unsigned n_out;
+
+    if ((ctl->sc_flags & SC_PACE) && pacer_delayed(&ctl->sc_pacer))
+        return 0;
+
+    cwnd = ctl->sc_ci->cci_get_cwnd(CGP(ctl));
+    n_out = send_ctl_all_bytes_out(ctl);
+    return n_out < cwnd;
+}
+
+
+void
+lsquic_send_ctl_maybe_app_limited (struct lsquic_send_ctl *ctl,
+                                            const struct network_path *path)
+{
+    const struct lsquic_packet_out *packet_out;
+
+    packet_out = lsquic_send_ctl_last_scheduled(ctl, PNS_APP, path, 0);
+    if ((packet_out && lsquic_packet_out_avail(packet_out) > 10)
+                                                || send_ctl_could_send(ctl))
+    {
+        LSQ_DEBUG("app-limited");
+        ctl->sc_flags |= SC_APP_LIMITED;
+    }
+}
+
+
 static void
 send_ctl_expire (struct lsquic_send_ctl *ctl, enum packnum_space pns,
                                                     enum expire_filter filter)
@@ -1806,6 +1849,7 @@ update_for_resending (lsquic_send_ctl_t *ctl, lsquic_packet_out_t *packet_out)
     packet_out->po_frame_types &= ~GQUIC_FRAME_REGEN_MASK;
     assert(packet_out->po_frame_types);
     packet_out->po_packno = packno;
+    lsquic_packet_out_set_ecn(packet_out, ctl->sc_ecn);
 
     if (ctl->sc_ver_neg->vn_tag)
     {
@@ -2116,9 +2160,14 @@ lsquic_send_ctl_drop_scheduled (lsquic_send_ctl_t *ctl)
 }
 
 
-static enum buf_packet_type
-send_ctl_determine_gquic_bpt (struct lsquic_send_ctl *ctl,
-                                        const struct lsquic_stream *stream)
+#ifdef NDEBUG
+static
+#elif __GNUC__
+__attribute__((weak))
+#endif
+enum buf_packet_type
+lsquic_send_ctl_determine_bpt (lsquic_send_ctl_t *ctl,
+                                            const lsquic_stream_t *stream)
 {
     const lsquic_stream_t *other_stream;
     struct lsquic_hash_elem *el;
@@ -2139,35 +2188,6 @@ send_ctl_determine_gquic_bpt (struct lsquic_send_ctl *ctl,
 }
 
 
-static enum buf_packet_type
-send_ctl_determine_ietf_bpt (struct lsquic_send_ctl *ctl,
-                                        const struct lsquic_stream *stream)
-{
-    if (lsquic_stream_is_critical(stream)
-            || stream == lsquic_prio_tree_highest_non_crit(
-                                    ctl->sc_conn_pub->u.ietf.prio_tree))
-        return BPT_HIGHEST_PRIO;
-    else
-        return BPT_OTHER_PRIO;
-}
-
-
-#ifdef NDEBUG
-static
-#elif __GNUC__
-__attribute__((weak))
-#endif
-enum buf_packet_type
-lsquic_send_ctl_determine_bpt (lsquic_send_ctl_t *ctl,
-                                            const lsquic_stream_t *stream)
-{
-    if (ctl->sc_flags & SC_IETF)
-        return send_ctl_determine_ietf_bpt(ctl, stream);
-    else
-        return send_ctl_determine_gquic_bpt(ctl, stream);
-}
-
-
 static enum buf_packet_type
 send_ctl_lookup_bpt (lsquic_send_ctl_t *ctl,
                                         const struct lsquic_stream *stream)
