@@ -29,6 +29,9 @@
 
 #define ADMIN_USERDB                "_AdminUserDB"
 
+#define TEST_CONF_LOG           DEFAULT_TMP_DIR "/testconf"
+
+
 extern const char *sStatDir;
 
 
@@ -50,6 +53,7 @@ class HttpConfigLoader;
 class VHostMap;
 class XmlNode;
 class ConfigCtx;
+class QuicEngine;
 
 class HttpServer : public TSingleton<HttpServer>, public HttpLogSource
 {
@@ -83,6 +87,9 @@ public:
     int mapListenerToVHost(const char *pListenerName,
                            const char *pKey,
                            const char *pVHostName);
+    int mapListenerToVHost(const char *pListenerName,
+                           HttpVHost    *pVHost,
+                           const char *pDomains);
     int mapListenerToVHost(HttpListener *pListener,
                            HttpVHost    *pVHost,
                            const char *pDomains);
@@ -138,6 +145,8 @@ public:
 //    void reconfigVHost( char *pVHostName, XmlNode* pRoot );
     void setServerRoot(const char *pRoot);
     int initServer(XmlNode *pRoot, int &iReleaseXmlTree, int reconfig = 0);
+    void addQuicEngine(QuicEngine *pEngine);
+    QuicEngine *getQuicEngine() const;
 
 
 };
