@@ -28,19 +28,10 @@ public:
     UnpackedHeaders();
     ~UnpackedHeaders();
 
+    static int convertHorQpackIdx(int hqpack_index);
     static int convertHpackIdx(int hpack_index);
 
-    int setMethod(const char *ptr, int len)
-    {
-        if (m_buf.size() != 4)
-            return LS_FAIL;
-        m_buf.append(ptr, len);
-        m_methodLen = len;
-        m_buf.append(" ", 1);
-        if (m_url.ptr)
-            appendUrl(m_url.ptr, m_url.len);
-        return 0;
-    }
+    int setMethod(const char *ptr, int len);
 
     int setUrl(const char *ptr, int len)
     {
@@ -79,7 +70,7 @@ public:
 
     int  appendReqLine(const char *method, int method_len,
                        const char *url, int url_len);
-    int appendHeader(int hpack_index, const char *name, int name_len,
+    int appendHeader(int index, const char *name, int name_len,
                      const char *val, int val_len);
     int appendCookieHeader(ls_str_t *cookies, int count, int total_len);
 
