@@ -23,6 +23,8 @@ class SslContextConfig;
 
 typedef SslContext *(*SslSniLookupCb)(void *arg, const char *pName);
 
+#define SSL_CTX_PENDING ((SSL_CTX *)-1L)
+
 struct bio_st;
 
 class SslContext
@@ -35,6 +37,7 @@ public:
         SSL_TLSv11  = 4,
         SSL_TLSv12  = 8,
         SSL_TLSv13  = 16,
+        SSL_TLS_SAFE = 28,
         SSL_TLS     = 30,
         SSL_ALL     = 31
     };
@@ -62,6 +65,10 @@ public:
     void setLastAccess(long t)  {   m_tmLastAccess = t;     }
 
     SSL_CTX *get() const        {   return m_pCtx;          }
+    void set(SSL_CTX * ctx)     {   m_pCtx = ctx;           }
+    
+    bool checkPrivateKey();
+
     SSL *newSSL();
     void setProtocol(int method);
 
